package cn.hanghui.outapi.platform.tenant.entity.enums;

import cn.hanghui.outapi.platform.common.utils.Preconditions;
import lombok.AllArgsConstructor;
import lombok.Getter;

/**
 * @author shisl
 * @package cn.hanghui.outapi.platform.tenant.entity.enums
 * @class AccountTypeEnum
 * @date 2024/7/1 下午2:26
 * @description 系统配置
 */
@Getter
@AllArgsConstructor
public enum SystemConfigEnum {

    /**
     * 系统配置
     * hideNavigationBar 隐藏底部导航栏
     * forbidPullDown 禁止下拉通知栏
     * autoLaunch 应用是否自启动
     * automaticEnterApplet 是否自动进入小程序
     * startPageUrl 启动页图片
     * homePageUrl 应用首页图片
     * mpassword 设备管理员密码
     * volume 语音音量
     */
    M_PASSWORD("M_PASSWORD", "设备管理员密码"),
    VOLUME("VOLUME", "语音音量(1~100)"),
    HIDE_NAVIGATION_BAR("HIDE_NAVIGATION_BAR", "隐藏底部导航栏"),
    FORBID_PULL_DOWN("FORBID_PULL_DOWN", "禁止下拉通知栏"),
    AUTO_LAUNCH("AUTO_LAUNCH", "应用是否自启动"),
    AUTOMATIC_ENTER_APPLET("AUTOMATIC_ENTER_APPLET", "是否自动进入小程序"),
    START_PAGE_URL("START_PAGE_URL", "启动页图片"),
    HOME_PAGE_URL("HOME_PAGE_URL", "应用首页图片");

    private String code;
    private String desc;

    public static SystemConfigEnum findByCode(String code) {
        if (Preconditions.isBlank(code)) {
            return null;
        }
        for (SystemConfigEnum enumObject : values()) {
            if (enumObject.code.equals(code)) {
                return enumObject;
            }
        }
        return null;
    }

}
